@@ -1484,13 +1484,27 @@ vk_video_get_h265_nal_unit(const StdVideoEncodeH265PictureInfo *pic_info)
    case STD_VIDEO_H265_PICTURE_TYPE_I:
       return HEVC_NAL_CRA_NUT;
    case STD_VIDEO_H265_PICTURE_TYPE_P:
-      return HEVC_NAL_TRAIL_R;
+      if (pic_info->TemporalId > 0)
+         if (pic_info->flags.is_reference)
+            return HEVC_NAL_TSA_R;
+         else
+            return HEVC_NAL_TSA_N;
+      else
+         if (pic_info->flags.is_reference)
+            return HEVC_NAL_TRAIL_R;
+         else
+            return HEVC_NAL_TRAIL_N;
    case STD_VIDEO_H265_PICTURE_TYPE_B:
       if (pic_info->flags.IrapPicFlag)
          if (pic_info->flags.is_reference)
             return HEVC_NAL_RASL_R;
          else
             return HEVC_NAL_RASL_N;
+      else if (pic_info->TemporalId > 0)
+         if (pic_info->flags.is_reference)
+            return HEVC_NAL_TSA_R;
+         else
+            return HEVC_NAL_TSA_N;
       else
           if (pic_info->flags.is_reference)
             return HEVC_NAL_TRAIL_R;
@@ -1714,16 +1728,16 @@ vk_video_encode_h264_pps(const StdVideoH264PictureParameterSet *pps,
 
 static void
 emit_nalu_h265_header(struct vl_bitstream_encoder *enc,
-                      int nal_unit_type)
+                      int nal_unit_type, int temporal_id)
 {
    enc->prevent_start_code = false;
 
    vl_bitstream_put_bits(enc, 24, 0);
    vl_bitstream_put_bits(enc, 8, 1);
    vl_bitstream_put_bits(enc, 1, 0);
-   vl_bitstream_put_bits(enc, 6, nal_unit_type); /* SPS NAL REF */
-   vl_bitstream_put_bits(enc, 6, 0);//nuh_layer_id
-   vl_bitstream_put_bits(enc, 3, 1);//nuh_temporal_id_plus1;
+   vl_bitstream_put_bits(enc, 6, nal_unit_type);   /* SPS NAL REF */
+   vl_bitstream_put_bits(enc, 6, 0);               /* nuh_layer_id */
+   vl_bitstream_put_bits(enc, 3, temporal_id + 1); /* nuh_temporal_id_plus1 */
    vl_bitstream_flush(enc);
 
    enc->prevent_start_code = true;
@@ -1766,7 +1780,7 @@ vk_video_encode_h265_vps(const StdVideoH265VideoParameterSet *vps,
 
    vl_bitstream_encoder_clear(&enc, data_ptr, data_size, size_limit);
 
-   emit_nalu_h265_header(&enc, HEVC_NAL_VPS_NUT);
+   emit_nalu_h265_header(&enc, HEVC_NAL_VPS_NUT, 0);
 
    vl_bitstream_put_bits(&enc, 4, vps->vps_video_parameter_set_id);
    vl_bitstream_put_bits(&enc, 2, 3);
@@ -1856,7 +1870,7 @@ vk_video_encode_h265_sps(const StdVideoH265SequenceParameterSet *sps,
 
    vl_bitstream_encoder_clear(&enc, data_ptr, data_size, size_limit);
 
-   emit_nalu_h265_header(&enc, HEVC_NAL_SPS_NUT);
+   emit_nalu_h265_header(&enc, HEVC_NAL_SPS_NUT, 0);
 
    vl_bitstream_put_bits(&enc, 4, sps->sps_video_parameter_set_id);
    vl_bitstream_put_bits(&enc, 3, sps->sps_max_sub_layers_minus1);
@@ -2014,7 +2028,7 @@ vk_video_encode_h265_pps(const StdVideoH265PictureParameterSet *pps,
 
    vl_bitstream_encoder_clear(&enc, data_ptr, data_size, size_limit);
 
-   emit_nalu_h265_header(&enc, HEVC_NAL_PPS_NUT);
+   emit_nalu_h265_header(&enc, HEVC_NAL_PPS_NUT, 0);
    vl_bitstream_exp_golomb_ue(&enc, pps->pps_pic_parameter_set_id);
    vl_bitstream_exp_golomb_ue(&enc, pps->pps_seq_parameter_set_id);
 
